
package org.daml.rdf.jena.common;

import org.daml.rdf.jena.model.Source;
import com.hp.hpl.mesa.rdf.jena.common.Store;
import com.hp.hpl.mesa.rdf.jena.model.*;
import java.util.Iterator;

/**
 * Interface to a triple store that keeps track of the source for each statement.
 *
 * @author der
 * @version $Id: SourceStore.java,v 1.1.1.1 2003/03/26 20:35:40 drager Exp $
 */
public interface SourceStore extends Store
{

    /**
     * Add a statement associated with a source.
     */
    public void add(Statement s, Source source) throws RDFException;

    /**
     * Return an iterator over all the statements with a given source.
     */
    public Iterator listBySource(Source source);

    /**
     * Return the store of a statement.
     */
    public Source getSource(Statement s);

    /**
     * Return an iterator over all the sources in the store.
     */
    public Iterator listSources();

    /**
     * Set the default source that is used when adding a statement.
     */
    public void setDefaultSource(Source source);

    /**
     * Get the default source that is used when adding a statement.
     */
    public Source getDefaultSource();

    /**
     * Change the source of a statement.
     */
    public void setSource(Statement s, Source source);

    /**
     * Create a new source.
     */
    public Source createSource(String uri);
    
}

