package org.daml.rdf.jena.mem;

import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.mem.*;
import org.daml.rdf.jena.model.DynamicModel;
import org.daml.rdf.jena.common.DynamicStore;
import org.daml.rdf.jena.model.Source;
import java.util.Set;


public class DynamicSourceModelMem extends SourceModelMem implements DynamicModel
{
    
    /**
     * URLs that have been read into this model
     */
    protected java.util.TreeSet urls = new java.util.TreeSet();

    /**
     * should we dynamically load URIs (not thread safe)
     */
    public static boolean dynamic = true;


    // ================================================================
    // Constructor methods 
    // ================================================================

    public DynamicSourceModelMem()
    {
	super();
	store = new DynamicSourceStoreMem(this);
    }

    public DynamicSourceModelMem(Source source)
    {
	super(source);
	store = new DynamicSourceStoreMem(source, this);
    }

    // ================================================================
    // DynamicModel methods
    // ================================================================

    /**
     * Enable or disable dynamic loading.
     * The dynamicStore must have a model in order to enable dynamic loading.
     * @returns the resulting value of dynamicLoad.
     */
    public boolean setDynamicLoad(boolean dynamicp)
    {
	((DynamicSourceStoreMem)store).setModel(this);
	return ((DynamicSourceStoreMem)store).setDynamicLoad(dynamicp);
    }

    /**
     * Returns true if dynamic loading is enabled.
     */
    public boolean dynamicLoad()
    {
	return ((DynamicSourceStoreMem)store).dynamicLoad();
    }


    /**
     * List which URLs have been loaded.
     * This is a cloned copy of the actual set of URLs.
     */

    public Set getLoadedURLs()
    {
	return (Set)urls.clone();
    }

    /**
     * Returns true if the given URL has been loaded into the model.
     */
    public boolean isLoaded(String url)
    { 
	return urls.contains(url);
    }


    public Model read(String url)
	throws RDFException
    {
	super.read(url);
	urls.add(url);
	return this;
    }


}
