package org.daml.rdf.jena.mem;

import org.daml.rdf.jena.model.Source;
import org.daml.rdf.jena.model.DynamicModel;
import org.daml.rdf.jena.common.DynamicResourceImpl;
import org.daml.rdf.jena.common.DynamicPropertyImpl;
import org.daml.rdf.jena.common.DynamicStore;
import org.daml.rdf.jena.common.SourceStore;
import org.daml.rdf.jena.common.SourceImpl;
import org.daml.rdf.jena.mem.StoreMem;
import com.hp.hpl.mesa.rdf.jena.common.Store;
import com.hp.hpl.mesa.rdf.jena.model.*;
//import com.hp.hpl.jena.rdf.arp.DynJenaReader;
//import com.hp.hpl.mesa.rdf.jena.mem.StoreMem;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Set;


/**
 * Implementation of a DynamicStore, as an extension of SourceStoreMem.
 *
 * @author drager
 * @version $Id: DynamicSourceStoreMem.java,v 1.1.1.1 2003/03/26 20:35:40 drager Exp $
 */
public class DynamicSourceStoreMem extends SourceStoreMem implements DynamicStore
{

    /**
     * A DynamicModel used to read in dynamically loaded files
     */
    protected DynamicModel model;

    /**
     * Flag that indicates if dynamic loading is enabled.
     */
    protected boolean dynamicLoadp = false;

    // ================================================================
    // Constructors
    // ================================================================

    public DynamicSourceStoreMem(DynamicModel model)
    {
	super();
	this.model = model;
	if (model != null) {
	    dynamicLoadp = true;
	    //DynJenaReader.useMe(this.model);
	} else {
	    dynamicLoadp = false;
	}
    }

    public DynamicSourceStoreMem() 
    {
	super();
	this.model = null;
	dynamicLoadp = false;
    }


    public DynamicSourceStoreMem(Source source, DynamicModel model)
    {
	super(source);
	this.model = model;
	if (model != null) {
	    dynamicLoadp = true;
	    //DynJenaReader.useMe(this.model);
	} else {
	    dynamicLoadp = false;
	}
    }

    public DynamicSourceStoreMem(Source source) 
    {
	super(source);
	this.model = null;
	dynamicLoadp = false;
    }
	   

    // ================================================================
    // DynamicStore methods
    // ================================================================

    /**
     * Enable or disable dynamic loading.
     * The dynamicStore must have a model in order to enable dynamic loading.
     * @returns true if the change was successful.
     */
    public boolean setDynamicLoad(boolean dynamicp)
    {
	if (dynamicp == true && model == null) {
	    dynamicLoadp = false;
	    return false;
	} else {
	    dynamicLoadp = dynamicp;
	    return dynamicLoadp;
	}
    }

    /**
     * Returns true if dynamic loading is enabled.
     */
    public boolean dynamicLoad()
    {
	return dynamicLoadp;
    }

    /**
     * Set the model for this store.
     * The model is used to execute the dynamic loading.
     */
    public void setModel(DynamicModel model)
    {
	this.model = model;
	//DynJenaReader.useMe(this.model);
    }

    /**
     * Return the model for this store.
     */
    public DynamicModel getModel()
    {
	return model;
    }

    // ================================================================
    // Overridden methods
    // ================================================================

    public Resource createResource(Model m) throws RDFException {
        return new DynamicResourceImpl(m);
    }
    
    public Resource createResource(String uri,Model m) throws RDFException {
        return new DynamicResourceImpl(uri, m);
    }
    
    public Property createProperty(String uri, Model m) throws RDFException {
          return new DynamicPropertyImpl(uri, m);
    }
    
    public Property createProperty(String namespace,String localPart,Model m)
      throws RDFException {
          return new DynamicPropertyImpl(namespace, localPart, m);
    }
    
    public Resource getResource(String uri,Model m) throws RDFException {
	Resource r = super.getResource(uri, m);
	if (r instanceof DynamicResourceImpl) {
	    return r;
	} else {
	    return new DynamicResourceImpl(uri, m);
	}
    }

    public Property getProperty(String uri, Model m)
      throws RDFException {
          return getProperty(createProperty(uri, m));
    }   
    
    public Property getProperty(String namespace, String localPart, Model m)
      throws RDFException {
        return getProperty(createProperty(namespace, localPart, m));
    }
    
    public Property getProperty(Property p) 
    {
        // check and see if we already have an object for this resource
        Statement s = predicates.getFirst(p);
        if (s != null) return s.getPredicate();
        
        // we don't so return the one we were passed
	return p;
    }

}
