package org.daml.rdf.jena.mem;

import org.daml.rdf.jena.model.DynamicModel;
import org.daml.rdf.jena.common.DynamicResourceImpl;
import org.daml.rdf.jena.common.DynamicPropertyImpl;
import org.daml.rdf.jena.common.DynamicStore;
import org.daml.rdf.jena.mem.StoreMem;

import com.hp.hpl.mesa.rdf.jena.common.Store;
import com.hp.hpl.mesa.rdf.jena.model.*;
//import com.hp.hpl.mesa.rdf.jena.mem.StoreMem;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Set;


/**
 * Implementation of a DynamicStore.
 *
 * @author drager
 * @version $Id: DynamicStoreMem.java,v 1.2 2003/03/26 22:39:19 drager Exp $
 */
public class DynamicStoreMem extends StoreMem implements DynamicStore
{

    /**
     * A DynamicModel used to read in dynamically loaded files
     */
    protected DynamicModel model;

    /**
     * Flag that indicates if dynamic loading is enabled.
     */
    protected boolean dynamicLoadp = false;

    // ================================================================
    // Constructors
    // ================================================================

    public DynamicStoreMem(DynamicModel model)
    {
	super();
	this.model = model;
	if (model != null) {
	    dynamicLoadp = true;
	} else {
	    dynamicLoadp = false;
	}
    }

    public DynamicStoreMem() 
    {
	super();
	this.model = null;
	dynamicLoadp = false;
    }


    // ================================================================
    // DynamicStore methods
    // ================================================================

    /**
     * Enable or disable dynamic loading.
     * The dynamicStore must have a model in order to enable dynamic loading.
     * @returns true if the change was successful.
     */
    public boolean setDynamicLoad(boolean dynamicp)
    {
	if (dynamicp == true && model == null) {
	    dynamicLoadp = false;
	    return false;
	} else {
	    dynamicLoadp = dynamicp;
	    return dynamicLoadp;
	}
    }

    /**
     * Returns true if dynamic loading is enabled.
     */
    public boolean dynamicLoad()
    {
	return dynamicLoadp;
    }

    /**
     * Set the model for this store.
     * The model is used to execute the dynamic loading.
     */
    public void setModel(DynamicModel model)
    {
	this.model = model;
    }

    /**
     * Return the model for this store.
     */
    public DynamicModel getModel()
    {
	return model;
    }

    /*
    Resource translate(ResourceImpl r) 
    {
	//***
	return new DynamicResourceImpl();
    }

    Property translate(Property p)
    {
	//***
	return new DynamicPropertyImpl();
    }
    */

    // ================================================================
    // Overridden methods from StoreMem
    // ================================================================

    /*
    public synchronized void add(Statement s)
	throws RDFException
    {
	boolean newRes = false;
	Resource newSub;
	Resource newProp;
	RDFNode newObj;

	Resource sub = s.getSubject();
	if ((sub instanceof ResourceImpl) &&
	    !(sub instanceof DynamicResourceImpl)) {
	    newSub = translate((ResourceImpl)sub);
	    newRes = true;
	} else {	
	    newSub = sub;
	}

	Property prop = s.getPredicate();
	if ((prop instanceof PropertyImpl) &&
	    !(sub instanceof DynamicPropertyImpl)) {
	    newProp = translate((PropertyImpl)prop);
	    newRes = true;
	} else {
	    newProp = prop;
	}

	RDFNode oNode = s.getObject();
	if ((oNode instanceOf ResourceImpl) &&
	    !(oNode instanceOf DynamicResourceImpl)) {
	    newObj = translate((ResourceImpl)oNode);
	    newRes = true;
	} else {
	    newObj = obj;
	}

	if (newRes = true) {
	    s = new createStatement(newSub, newProp, newObj);
	}
	
	super.add(s);
    }

    public Resource getResource(String uri,Model m) throws RDFException {
	Resource r = super.getResource(uri, m);
	if (r instanceof DynamicResourceImpl) {
	    return r;
	} else {
	    return new DynamicResourceImpl(uri, m);
	}
    }

    public Property getProperty(Property p) 
    {
	if (!(p instanceof DynamicPropertyImpl)) {
	    p = translate(p);
	}
        // check and see if we already have an object for this resource
        Statement s = predicates.getFirst(p);
        if (s != null) return s.getPredicate();
        
        // we don't so return the one we were passed
	return p;
    }    
    */

    // ================================================================
    // Overridden methods from StoreCom
    // ================================================================

    public Resource createResource(Model m) throws RDFException {
        return new DynamicResourceImpl(m);
    }
    
    public Resource createResource(String uri,Model m) throws RDFException {
        return new DynamicResourceImpl(uri, m);
    }
    
    public Property createProperty(String uri, Model m) throws RDFException {
          return new DynamicPropertyImpl(uri, m);
    }
    
    public Property createProperty(String namespace,String localPart,Model m)
      throws RDFException {
          return new DynamicPropertyImpl(namespace, localPart, m);
    }
    
    public Property getProperty(String uri, Model m)
      throws RDFException {
          return getProperty(createProperty(uri, m));
    }   
    
    public Property getProperty(String namespace, String localPart, Model m)
      throws RDFException {
        return getProperty(createProperty(namespace, localPart, m));
    }
    


}
