package org.daml.rdf.jena.model;

import com.hp.hpl.mesa.rdf.jena.model.*;

/**
 * An RDF Model that provides a mechanism to dynamically load
 * referenced namespaces on an as needed basis. When the properties of
 * a Resource are accessed the model checks to see if the namespace of
 * the resource has been loaded into the model. If not, the namespace
 * is loaded before the accessor method returns. This model makes the
 * assumption that the namespace of a resource is the URL of the file
 * that defines the resource. <i>Please be careful when using
 * StmtIterators on a DynamicModel because a dynamic load modifes the
 * model supporting the iterator which causes an error.</i>

 *
 * @author der
 * @version $Id: DynamicModel.java,v 1.2 2003/03/26 22:39:19 drager Exp $
 */
public interface DynamicModel extends Model
{

    /**
     * Enable or disable dynamic loading.
     * The dynamicStore must have a model in order to enable dynamic loading.
     * @returns the resulting value of dynamicLoad.
     */
    public boolean setDynamicLoad(boolean dynamicp);

    /**
     * Returns true if dynamic loading is enabled.
     */
    public boolean dynamicLoad();
    

    /**
     * Provide a function that determines what to load for a given resource.
     * TBD
     */
    // public void setDynamicLoadFunction(String function);
    
    /**
     * List which URLs have been loaded.
     */
    public java.util.Set getLoadedURLs();

    /**
     * Returns true if the given URL has been loaded into the model.
     */
    public boolean isLoaded(String url);

}
