package org.daml.rdf.jena.model;

import com.hp.hpl.mesa.rdf.jena.model.*;
import java.util.Iterator;
import java.io.Reader;

/**
 * An RDF Model that keeps statements grouped by source.
 *
 * @author der
 * @version $Id: SourceModel.java,v 1.2 2003/03/26 22:39:19 drager Exp $
 */
public interface SourceModel extends Model
{

    /**
     * Add all the statements from another model to this model and associate them with a source.
     */
    Model add(Model m, Source source) throws RDFException;

    /**
     * Add a statement to this model and associate it with a source.
     */
    Model add(Statement s, Source source) throws RDFException;

    /**
     * Add all the statements in an interator to this model and associate them with a source.
     */
    Model add(StmtIterator iter, Source source) throws RDFException;

    /**
     * List all sources with statements in the model.
     */
    Iterator listSources() throws RDFException;
    
    /**
     * Return a statement iterator over all the statements associated with a source.
     */
    StmtIterator listStatementsOfSource(Source source);

    /**
     * Return a new model that contains all the statements associated with a source.
     */
    Model getModelForSource(Source source) throws RDFException;

    /**
     * Change the source that is associated with a statement.
     */
    Model setSource(Statement s, Source source);

    /**
     * Change the source associated with each of the statements in the iterator.
     */
    Model setSource(StmtIterator iter, Source source) throws RDFException;

    /**
     * Write out to a file just the statements associated with a source.
     * The source information is not stored with the statements in the output.
     */
    Model write(java.io.Writer writer, Source source) throws RDFException; 

    /**
     * Write out to a file just the statements associated with a source.
     * The source information is not stored with the statements in the output.
     */
    Model write(java.io.Writer writer, java.lang.String lang, Source source) throws RDFException;

    /**
     * Write out to a file just the statements associated with a source.
     * The source information is not stored with the statements in the output.
     */
    Model write(java.io.Writer writer, java.lang.String lang, java.lang.String base, Source source) throws RDFException;

    /**
     * Read a source file into the model and assign the new statments to a source.
     */
    public Model read(Reader reader, String base, String lang, Source source) throws RDFException;

    /**
     * Read a source file into the model and assign the new statments to a source.
     */
    public Model read(Reader reader, String base, Source source) throws RDFException;

    /**
     * Set the default source for any new statements that are added to the model.
     * @returns the old default source.
     */
    public Source setDefaultSource(Source source);

    /**
     * Get the default source that will be assigned to any new statements added to the model.
     */
    public Source getDefaultSource();

    /**
     * Create a new source object from a URI.
     */
    public Source createSource(String uri);

}
